package es.uva.tfg.geolocalizationgame.controllers;

import java.util.Date;

import javax.validation.ConstraintViolationException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import es.uva.tfg.geolocalizationgame.exceptions.JuegoNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.MalformedRequestException;
import es.uva.tfg.geolocalizationgame.exceptions.ModalidadDeJuegoNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.NivelDificultadNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.UserNotFoundException;
import es.uva.tfg.geolocalizationgame.resources.ErrorMessage;
import es.uva.tfg.geolocalizationgame.resources.Juego;
import es.uva.tfg.geolocalizationgame.resources.ModalidadDeJuego;
import es.uva.tfg.geolocalizationgame.resources.NivelDificultad;
import es.uva.tfg.geolocalizationgame.resources.Partida;
import es.uva.tfg.geolocalizationgame.resources.Usuario;
import es.uva.tfg.geolocalizationgame.services.JuegoService;
import es.uva.tfg.geolocalizationgame.services.ModalidadDeJuegoService;
import es.uva.tfg.geolocalizationgame.services.NivelDificultadService;
import es.uva.tfg.geolocalizationgame.services.PartidaService;
import es.uva.tfg.geolocalizationgame.services.UserService;

/**
 * Controlador Rest para la entidad Partida.
 * 
 * @author Cristina Gomez Escribano
 * @version 2.0
 * @since 18/12/2018
 *
 */
@RestController
@RequestMapping("/juegos/{idJuego}/partidas")
public class PartidaRestController {

	@Autowired
	private UserService userService;
	
	@Autowired
	private JuegoService juegoService;
	
	@Autowired
	private PartidaService partidaService;
	
	@Autowired
	private ModalidadDeJuegoService modoJuegoService;
	
	@Autowired
	private NivelDificultadService nivelDificultadService;
	
	/**
	 * Recibe la peticion para crear una nueva partida de un juego.
	 * 
	 * @param idJuego
	 * @param partida
	 * @return
	 * @throws MalformedRequestException
	 * @throws JuegoNotFoundException
	 * @throws UserNotFoundException
	 * @throws ModalidadDeJuegoNotFoundException
	 * @throws NivelDificultadNotFoundException
	 */
	@RequestMapping(method = RequestMethod.POST, consumes=MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Void> postPartida(@PathVariable Integer idJuego, @RequestBody Partida partida) throws MalformedRequestException, JuegoNotFoundException, UserNotFoundException, ModalidadDeJuegoNotFoundException, NivelDificultadNotFoundException {
		try {
			Juego juego= juegoService.obtenerJuegoById(idJuego);
			if(juego==null) {
				throw new JuegoNotFoundException();
			}
	
			Authentication auth = SecurityContextHolder.getContext().getAuthentication();
			Usuario userAuth=userService.obtenerUsuarioByLogin(auth.getName());
			if(userAuth==null) {
				throw new UserNotFoundException();
			}
			
			ModalidadDeJuego modoJuego = modoJuegoService.obtenerModalidadDeJuegoByName(partida.getModalidadJuego().getModalidad());
			if(modoJuego==null) {//no existe la modalidad
				throw new ModalidadDeJuegoNotFoundException();
			}
			
			NivelDificultad nivel = nivelDificultadService.obtenerNivelDificultadByNombre(partida.getNivelDificultad().getNivelDificultad());
			if(nivel==null) {//no existe el nivel de dificultad
				throw new NivelDificultadNotFoundException();
			}
			
			partida.setCreador(userAuth);
			partida.setJuego(juego);
			partida.setModalidadJuego(modoJuego);
			partida.setNivelDificultad(nivel);
			partida.setFechaCreacion(new Date());
			partidaService.savePartida(partida);
			userService.actualizarPuntosConseguidos(userAuth, partida.getPuntosAcertados());
			userService.updateUserData(userAuth);
			return new ResponseEntity<Void>(HttpStatus.CREATED);
			
		} catch(ConstraintViolationException ex) {
			throw new MalformedRequestException(ErrorMessage.getErrores(ex.getConstraintViolations()));
		}
	}
	
}
