package es.uva.tfg.geolocalizationgame.controllers;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.fasterxml.jackson.annotation.JsonView;

import es.uva.tfg.geolocalizationgame.exceptions.JuegoNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.PuntoGeograficoExistException;
import es.uva.tfg.geolocalizationgame.exceptions.PuntoGeograficoIsLastException;
import es.uva.tfg.geolocalizationgame.exceptions.PuntoGeograficoNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.PuntoGeograficoNotInGameException;
import es.uva.tfg.geolocalizationgame.exceptions.TipoPuntoNotFoundException;
import es.uva.tfg.geolocalizationgame.exceptions.UserNotCreatorOfJuegoException;
import es.uva.tfg.geolocalizationgame.exceptions.UserNotFoundException;
import es.uva.tfg.geolocalizationgame.resources.DatosUser;
import es.uva.tfg.geolocalizationgame.resources.Juego;
import es.uva.tfg.geolocalizationgame.resources.PuntoGeografico;
import es.uva.tfg.geolocalizationgame.resources.TipoPunto;
import es.uva.tfg.geolocalizationgame.resources.Usuario;
import es.uva.tfg.geolocalizationgame.services.JuegoService;
import es.uva.tfg.geolocalizationgame.services.PuntoGeograficoService;
import es.uva.tfg.geolocalizationgame.services.TipoPuntoService;
import es.uva.tfg.geolocalizationgame.services.UserService;

/**
 * Controlador Rest para la entidad PuntoGeografico.
 * 
 * @author Cristina Gomez Escribano
 * @version 3.0
 * @since 03/10/2018
 *
 */
@RestController
@RequestMapping("/juegos/{idJuego}/puntos")
public class PuntoGeograficoRestController {

	@Autowired
	private JuegoService juegoService;
	
	@Autowired
	private PuntoGeograficoService puntoService;
	
	@Autowired
	private TipoPuntoService tipoService;
	
	@Autowired
	private UserService userService;
	
	/**
	 * Recibe la peticion para añadir un nuevo punto geografico a un juego.
	 * 
	 * @param idJuego
	 * @param punto
	 * @return
	 * @throws JuegoNotFoundException
	 * @throws TipoPuntoNotFoundException
	 * @throws PuntoGeograficoExistException
	 * @throws UserNotFoundException
	 * @throws UserNotCreatorOfJuegoException
	 */
	@RequestMapping(method = RequestMethod.POST, consumes=MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<Void> postPuntoGeografico(@PathVariable Integer idJuego, @RequestBody PuntoGeografico punto) throws JuegoNotFoundException, TipoPuntoNotFoundException, PuntoGeograficoExistException, UserNotFoundException, UserNotCreatorOfJuegoException {
		Juego juego= juegoService.obtenerJuegoById(idJuego);
		if(juego==null) {
			throw new JuegoNotFoundException();
		}
		TipoPunto tipo = tipoService.obtenerTipoPuntoByNombre(punto.getIdTipoPunto().getTipoPunto());
		if(tipo==null) {
			throw new TipoPuntoNotFoundException();
		}
		punto.setIdTipoPunto(tipo);
		punto.setJuego(juego);
		
		Authentication auth = SecurityContextHolder.getContext().getAuthentication();
		Usuario userAuth=userService.obtenerUsuarioByLogin(auth.getName());
		if(userAuth==null) {
			throw new UserNotFoundException();
		}
		if(!juego.getCreador().equals(userAuth)) { //comprobar creador del juego
			throw new UserNotCreatorOfJuegoException();
		}
		if(puntoService.existPuntoGeografico(juego, punto)) {
			throw new PuntoGeograficoExistException();
		}
		puntoService.savePunto(punto);	
		return new ResponseEntity<Void>(HttpStatus.CREATED);
	}
	
	/**
	 * Peticion para obtener los puntos geograficos de un juego concreto.
	 * 
	 * @param idJuego
	 * @return ResponseEntity<List<PuntoGeografico>>
	 * @throws JuegoNotFoundException
	 */
	@JsonView(DatosUser.GameView.class)
	@RequestMapping(method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<List<PuntoGeografico>> getPuntosGeograficosByJuego(@PathVariable Integer idJuego) throws JuegoNotFoundException {
		Juego juego= juegoService.obtenerJuegoById(idJuego);
		if(juego==null) {
			throw new JuegoNotFoundException();
		}
		return new ResponseEntity<List<PuntoGeografico>>(puntoService.obtenerPuntosGeograficosByJuego(juego), HttpStatus.OK);
	}
	
	/**
	 * Recibe la peticion para eliminar un punto geografico de un juego por id de punto.
	 * 
	 * @param idJuego
	 * @param idPunto
	 * @return ResponseEntity<Void>
	 * @throws JuegoNotFoundException
	 * @throws PuntoGeograficoNotFoundException
	 * @throws UserNotFoundException
	 * @throws UserNotCreatorOfJuegoException
	 * @throws PuntoGeograficoNotInGameException
	 * @throws PuntoGeograficoIsLastException 
	 */
	@RequestMapping(value="{idPunto}", method = RequestMethod.DELETE)
	public ResponseEntity<Void> deletePuntoGeografico(@PathVariable Integer idJuego, @PathVariable Integer idPunto) throws JuegoNotFoundException, PuntoGeograficoNotFoundException, UserNotFoundException, UserNotCreatorOfJuegoException, PuntoGeograficoNotInGameException, PuntoGeograficoIsLastException {
		Juego juego = juegoService.obtenerJuegoById(idJuego);
		if(juego==null) {
			throw new JuegoNotFoundException();
		}
		PuntoGeografico punto = puntoService.obtenerPuntoGeograficoById(idPunto);
		if(punto==null) {
			throw new PuntoGeograficoNotFoundException();
		}
		Authentication auth = SecurityContextHolder.getContext().getAuthentication();
		Usuario userAuth=userService.obtenerUsuarioByLogin(auth.getName());
		if(userAuth==null) {
			throw new UserNotFoundException();
		}
		if(!juego.getCreador().equals(userAuth)) { //comprobar creador del juego
			throw new UserNotCreatorOfJuegoException();
		}
		if(!punto.getJuego().equals(juego)) {//comprobar que el punto sea del juego
			throw new PuntoGeograficoNotInGameException();
		}
		if(puntoService.obtenerPuntosGeograficosByJuego(juego).size()<=1) {//comprobar que no sea el ultimo punto del juego
			throw new PuntoGeograficoIsLastException();
		}
		//todo correcto, se elimina el punto. Detalle de solo 1 punto no funciona
		puntoService.eliminarPuntoGeografico(punto);
		return new ResponseEntity<Void>(HttpStatus.OK);
	}
}
