package es.uva.tfg.geolocalizationgame.services;

import java.util.List;

import javax.validation.Valid;

import es.uva.tfg.geolocalizationgame.exceptions.TipoVisibilidadNotFoundException;
import es.uva.tfg.geolocalizationgame.resources.Juego;
import es.uva.tfg.geolocalizationgame.resources.Usuario;

/**
 * Implementa la interfaz del servicio para la entidad Juego.
 * 
 * @author Cristina Gomez Escribano
 * @since 10/11/2018
 * @version 2.0
 *
 */
public interface JuegoService {

	/**
	 * Devuelve un juego por id.
	 * 
	 * @param idJuego
	 * @return Juego
	 */
	Juego obtenerJuegoById(Integer idJuego);

	/**
	 * Elimina un juego pasado como parametro.
	 * 
	 * @param juego
	 */
	void eliminarJuego(@Valid Juego juego);

	/**
	 * Obtiene los juegos creados por un usuario.
	 * 
	 * @param usuario
	 * @return List<Juego>
	 */
	List<Juego> obtenerJuegosByLogin(@Valid Usuario usuario);

	/**
	 * Devuelve todos los juegos publicos que contienen un texto pasado como parametro.
	 * 
	 * @param texto
	 * @return List<Juego>
	 */
	List<Juego> obtenerJuegosPublicosByText(String texto);

	/**
	 * Devuelve si un juego esta disponible para un usuario concreto.
	 * 
	 * @param usuario
	 * @param juego
	 * @return boolean
	 */
	boolean compruebaJuegoPermitido(@Valid Usuario usuario, @Valid Juego juego);

	/**
	 * Actualiza el nombre de un juego.
	 * 
	 * @param juego
	 * @param nombre
	 */
	void actualizarNombre(@Valid Juego juego, String nombre);

	/**
	 * Realiza la llamada para actualizar el juego en el sistema.
	 * 
	 * @param juego
	 * @return Juego
	 */
	Juego updateJuego(@Valid Juego juego);

	/**
	 * Devuelve un juego generado con los datos que se le pasan como parametro.
	 * 
	 * @param usuario
	 * @param nombreJuego
	 * @return Juego
	 * @throws TipoVisibilidadNotFoundException
	 */
	Juego crearJuego(@Valid Usuario usuario, String nombreJuego) throws TipoVisibilidadNotFoundException;

	/**
	 * Realiza la llamada para persistir un juego en el sistema.
	 * 
	 * @param juegoCreado
	 */
	void saveJuego(@Valid Juego juegoCreado);

	/**
	 * Actualiza la visibilidad de un juego.
	 * 
	 * @param juego
	 * @param visibilidad
	 * @return boolean
	 */
	boolean actualizarVisibilidad(@Valid Juego juego, Integer visibilidad);

}
