package es.uva.tfg.geolocalizationgame.services;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import es.uva.tfg.geolocalizationgame.exceptions.TipoVisibilidadNotFoundException;
import es.uva.tfg.geolocalizationgame.persistence.LogroRepository;
import es.uva.tfg.geolocalizationgame.resources.Logro;
import es.uva.tfg.geolocalizationgame.resources.LogroPK;
import es.uva.tfg.geolocalizationgame.resources.PuntoGeografico;
import es.uva.tfg.geolocalizationgame.resources.TipoLogro;
import es.uva.tfg.geolocalizationgame.resources.TipoPunto;
import es.uva.tfg.geolocalizationgame.resources.TipoVisibilidad;
import es.uva.tfg.geolocalizationgame.resources.Usuario;


/**
 * Implementa la clase de servicio para la entidad Logro.
 * 
 * @author Cristina Gómez Escribano
 * @version 2.0
 * @since 16/09/2018
 */
@Service
public class LogroServiceImpl implements LogroService {

	@Autowired
	private LogroRepository logroRepository;
	
	@Autowired
	private TipoLogroService tipoLogroService;
	
	@Autowired
	private TipoVisibilidadService tipoVisibilidadService;

	@Override
	public Logro obtenerLogroById(Usuario usuario, TipoLogro tipoLogro) {
		LogroPK idLogro = new LogroPK(usuario, tipoLogro);
		return logroRepository.findOne(idLogro);
	}

	@Override
	public boolean actualizarVisibilidad(Logro logro, Integer visibilidad) {
		boolean update=false;
		TipoVisibilidad tipoVisibilidad=tipoVisibilidadService.obtenerVisibilidadById(visibilidad);
		if(tipoVisibilidad!=null) {//se ha encontrado, por tanto se actualiza
			logro.setTipoVisibilidad(tipoVisibilidad);
			logro.setFechaModificacion(new Date());
			update=true;
		}
		return update;
	}

	@Override
	public Logro updateLogro(Logro logro) {
		return logroRepository.saveAndFlush(logro);
	}

	@Override
	public List<Logro> obtenerLogrosNuevos(Usuario usuario) throws TipoVisibilidadNotFoundException {
		List<Logro> logrosNuevos= new ArrayList<>();
		List<TipoLogro> logrosAComprobar=obtenerLogrosAComprobar(usuario.getLogros());
		List<PuntoGeografico> puntosConseguidos=usuario.getPuntosConseguidos();
		Logro logroConseguido;
		
		for(int i=0; i<logrosAComprobar.size(); i++) {//recorrer cada logro
			if(comprobarLogro(logrosAComprobar.get(i), puntosConseguidos)) {//se ha conseguido el logro
				logroConseguido = crearLogro(usuario, logrosAComprobar.get(i));
				if(logroConseguido!=null) {
					saveLogro(logroConseguido);
					logrosNuevos.add(logroConseguido);
				}
			}
		}
		return logrosNuevos;
	}
	
	/**
	 * Devuelve si un logro se ha conseguido o no.
	 * 
	 * @param logro
	 * @param puntosConseguidos
	 * @return conseguido
	 */
	private boolean comprobarLogro(TipoLogro logro, List<PuntoGeografico> puntosConseguidos) {
		boolean conseguido=false;
		String continente=logro.getContinente();
		String pais=logro.getPais();
		Integer numPuntos=logro.getPuntosAConseguir();
		TipoPunto tipoPunto=logro.getIdTipoPunto();

		Integer numPuntosConseguidos=0;
		
		if(puntosConseguidos.size()>=numPuntos) {//si se han conseguido menos puntos de los que se requieren para el logro, no estara conseguido
			for(int j=0; j<puntosConseguidos.size();j++) {
				if(continente!=null && puntosConseguidos.get(j).getContinente().equalsIgnoreCase(continente)) {//comprobar si es logro de continente
					numPuntosConseguidos+=1;
				}else if(pais!=null && puntosConseguidos.get(j).getPais().equalsIgnoreCase(pais)){//comprobar si es logro de pais
					numPuntosConseguidos+=1;
				}else if(tipoPunto!=null && puntosConseguidos.get(j).getIdTipoPunto().equals(tipoPunto)) {//comprobar si es logro de tipo de punto
					numPuntosConseguidos+=1;
				}
			}
			//comprobar si coincide o es superior a los puntos requeridos del logro
			if(numPuntosConseguidos>=numPuntos) {//se ha cumplido el logro
				conseguido=true;
			}
		}
		return conseguido;
	}
	
	/**
	 * Devuelve un logro nuevo.
	 * 
	 * @param usuario
	 * @param tipoLogro
	 * @return logro
	 * @throws TipoVisibilidadNotFoundException
	 */
	private Logro crearLogro(Usuario usuario, TipoLogro tipoLogro) throws TipoVisibilidadNotFoundException {
		TipoVisibilidad visibilidad=tipoVisibilidadService.obtenerVisibilidadByNombre("privado");
		if(visibilidad==null) {
			throw new TipoVisibilidadNotFoundException();
		}
		Logro logro=new Logro();
		LogroPK idLogro= new LogroPK();
		idLogro.setUsuario(usuario);
		idLogro.setTipoLogro(tipoLogro);
		logro.setId(idLogro);
		logro.setFechaConseguido(new Date());
		logro.setFechaModificacion(new Date());
		logro.setTipoVisibilidad(visibilidad);
		return logro;
		
	}

	/**
	 * Devuelve los logros no conseguidos por un usuario.
	 * 
	 * @param logrosConseguidos
	 * @return
	 */
	public List<TipoLogro> obtenerLogrosAComprobar(List<Logro> logrosConseguidos){
		boolean conseguido=false;
		List<TipoLogro> logrosAComprobar=new ArrayList<>();
		List<TipoLogro> logros = tipoLogroService.obtenerLogros();
		for (int i=0; i<logros.size(); i++) {
			conseguido=false;
			for(int j=0; j<logrosConseguidos.size();j++){
				if(logros.get(i).getIdLogro().equals(logrosConseguidos.get(j).getId().getTipoLogro().getIdLogro())) {
					conseguido=true;
					break;
				}
			}
			
			if(!conseguido) {//si no lo he conseguido, se mete para comprobar
				logrosAComprobar.add(logros.get(i));
			}
		}
		return logrosAComprobar;
	}

	@Override
	public void saveLogro(Logro logro) {
		logroRepository.saveAndFlush(logro);	
	}
}
