package es.uva.tfg.geolocalizationgame.services;

import java.util.ArrayList;
import java.util.List;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import es.uva.tfg.geolocalizationgame.persistence.UsuarioRepository;
import es.uva.tfg.geolocalizationgame.resources.Juego;
import es.uva.tfg.geolocalizationgame.resources.Logro;
import es.uva.tfg.geolocalizationgame.resources.MapeoUser;
import es.uva.tfg.geolocalizationgame.resources.PuntoGeografico;
import es.uva.tfg.geolocalizationgame.resources.Usuario;

/**
 * Implementación del servicio para la entidad usuario
 * 
 * @author Cristina Gómez Escribano
 * @version 3.0
 * @since 02/05/2018
 *
 */
@Service
public class UserServiceImpl implements UserService {

	@Autowired
	private UsuarioRepository usuarioRepository;

	@Autowired
	private PasswordEncoder passwordEncoder;

	@Override
	public boolean altaUsuario(@Valid Usuario usuario) {

		Usuario user = usuarioRepository.findOne(usuario.getLogin());
		if (user == null) {// no existe en la BBDD
			usuario.setPassword(passwordEncoder.encode(usuario.getPassword()));// guardamos el password encriptado
			usuarioRepository.save(usuario);
			return true;
		} else {
			return false;
		}
	}

	@Override
	public Usuario obtenerUsuarioByLogin(String login) {
		return usuarioRepository.findOne(login);
	}

	@Override
	public Usuario updateUserData(@Valid Usuario usuario) {
		return usuarioRepository.saveAndFlush(usuario);
	}

	@Override
	public boolean actualizarDatosUsuario(MapeoUser usuario, Usuario userCurrent) {
		boolean resultado = false;
		if ((usuario.getCurrentPassword() == "" || usuario.getCurrentPassword() == null)
				&& (usuario.getNewPassword() == "" || usuario.getNewPassword() == null)) { // no se ha cambiado la pass
			userCurrent.setAvatar(usuario.getAvatar());
			userCurrent.setEmail(usuario.getEmail());
			userCurrent.setNombre(usuario.getNombre());
			resultado = true;
		} else {// se ha cambiado la pass
			if (!passwordEncoder.matches(usuario.getCurrentPassword(), userCurrent.getPassword())) {// no coinciden las
																									// pass, error
				resultado = false;
			} else {
				userCurrent.setPassword(passwordEncoder.encode(usuario.getNewPassword()));
				resultado = true;
			}
		}
		return resultado;
	}

	@Override
	public List<Usuario> getUsuarios(String text) {
		List<Usuario> usuarios;
		if (text == null || text == "") {// se devuelven todos los usuarios
			usuarios = usuarioRepository.findAll();

		} else {
			usuarios = usuarioRepository.obtenerUsuariosByText(text.toUpperCase());
		}
		deleteUserAuth(usuarios);
		setPasswordToEmpty(usuarios);
		return usuarios;
	}

	@Override
	public boolean agregarAmigo(Usuario usuario, Usuario newAmigo) {
		boolean update = true;
		List<Usuario> amigos = usuario.getAmigos();
		List<Usuario> amigoDe = usuario.getAmigosDe();
		for (int i = 0; i < amigos.size(); i++) {
			if (amigos.get(i).getLogin().equals(newAmigo.getLogin())) {// ya son amigos
				update = false;
			}
		}
		if (update) {//agrego el nuevo amigo
			amigos.add(newAmigo);
			amigoDe.add(newAmigo);
			usuario.setAmigos(amigos);
			usuario.setAmigosDe(amigoDe);
		}
		return update;
	}
	
	@Override
	public boolean eliminarAmigo(Usuario usuario, Usuario newAmigo) {
		boolean eliminar = false;
		List<Usuario> amigos = usuario.getAmigos();
		List<Usuario> amigoDe = usuario.getAmigosDe();
		for (int i = 0; i < amigos.size(); i++) {
			if (amigos.get(i).getLogin().equals(newAmigo.getLogin())) {// lo elimino de la lista
				amigos.remove(i);
				amigoDe.remove(i);
				eliminar = true;
			}
		}
		return eliminar;
	}

	/**
	 * Quitar de la lista el usuario autenticado, que es el que está buscando
	 * 
	 * @param usuarios
	 */
	public void deleteUserAuth(List<Usuario> usuarios) {
		Authentication auth = SecurityContextHolder.getContext().getAuthentication();
		Usuario userAuth = obtenerUsuarioByLogin(auth.getName());
		for (int i = 0; i < usuarios.size(); i++) {
			if (usuarios.get(i).getLogin().equals(userAuth.getLogin())) {// es el usuario autenticado lo elimino
				usuarios.remove(i);
			}
		}
	}

	/**
	 * Vaciar la contraseña para evitar problemas de seguridad
	 * 
	 * @param usuarios
	 */
	public void setPasswordToEmpty(List<Usuario> usuarios) {
		for (int i = 0; i < usuarios.size(); i++) {
			usuarios.get(i).setPassword("");
		}
	}

	@Override
	public void actualizarLogrosAmigos(Usuario usuario) {
		List<Usuario> amigos=usuario.getAmigos();
		for(int i=0; i<amigos.size();i++) {//recorrer amigos
			List<Logro> logros=new ArrayList<Logro>();
			for(int j=0; j<amigos.get(i).getLogros().size(); j++) {//recorrer logros de cada amigo
				if(amigos.get(i).getLogros().get(j).getTipoVisibilidad().getIdTipoVisibilidad()==2 || amigos.get(i).getLogros().get(j).getTipoVisibilidad().getIdTipoVisibilidad()==3) {//si el logro esta en modo privado, lo elimino
					logros.add(amigos.get(i).getLogros().get(j));
				}
			}
			amigos.get(i).setLogros(logros);
		}
		usuario.setAmigos(amigos);
	}
	
	@Override
	public void actualizarJuegosAmigos(Usuario usuario) {
		List<Usuario> amigos=usuario.getAmigos();
		for(int i=0; i<amigos.size();i++) {//recorrer amigos
			List<Juego> juegos=new ArrayList<Juego>();
			for(int j=0; j<amigos.get(i).getJuegos().size(); j++) {//recorrer juegos de cada amigo
				if(amigos.get(i).getJuegos().get(j).getVisibilidad().getIdTipoVisibilidad()==2 || amigos.get(i).getJuegos().get(j).getVisibilidad().getIdTipoVisibilidad()==3) {//si el juego esta en modo privado, lo elimino
					juegos.add(amigos.get(i).getJuegos().get(j));
				}
			}
			amigos.get(i).setJuegos(juegos);
		}
		usuario.setAmigos(amigos);
	}

	@Override
	public void actualizarLogrosUsuario(Usuario usuario) {
		List<Logro> logros=new ArrayList<Logro>();
		for(int i=0; i<usuario.getLogros().size();i++) {
			if(usuario.getLogros().get(i).getTipoVisibilidad().getIdTipoVisibilidad()==2 || usuario.getLogros().get(i).getTipoVisibilidad().getIdTipoVisibilidad()==3) {//elimino los logros que sean privados
				logros.add(usuario.getLogros().get(i));
			}
		}
		usuario.setLogros(logros);
	}
	
	@Override
	public void actualizarJuegosUsuario(Usuario usuario) {
		List<Juego> juegos=new ArrayList<Juego>();
		for(int i=0; i<usuario.getJuegos().size();i++) {
			if(usuario.getJuegos().get(i).getVisibilidad().getIdTipoVisibilidad()==2 || usuario.getJuegos().get(i).getVisibilidad().getIdTipoVisibilidad()==3) {//elimino los juegos que sean privados
				juegos.add(usuario.getJuegos().get(i));
			}
		}
		usuario.setJuegos(juegos);
	}

	@Override
	public void actualizarPuntosConseguidos(@Valid Usuario usuario, List<PuntoGeografico> puntosNuevos) {
		boolean encontrado=false;
		List<PuntoGeografico> puntosConseguidos = usuario.getPuntosConseguidos();
		for(int i=0; i<puntosNuevos.size(); i++) {
			encontrado=false;
			for(int j=0; j<puntosConseguidos.size(); j++) {
				if(puntosNuevos.get(i).getIdPunto().equals(puntosConseguidos.get(j).getIdPunto())) {
					encontrado=true;
					break;
				}
			}
			if(!encontrado) {
				puntosConseguidos.add(puntosNuevos.get(i));
			}
			
		}
		
	}
}
